use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Header;

use Spiffy -Base;

field 'values';

sub new {
	return super(values => [@_])
}

sub istoken() {
	return $_[0] =~ /^[!\#-'*.0-9A-Z^_\`a-z|-]+$/;
}

sub isnasty() {
	return $_[0] =~ /[^ -~]|^\s|\s$|\s\s|[;"]/;
}

sub toString {
	my @values = @{$self->values};

	my @out;

	push @out, shift @values
		if @values & 1;
	
	while(@values) {
		my $out = '';
		my $key = shift @values;
		my $val = shift @values;
		if(defined $key) {
			die "Malformed header token '$key'\n"
				unless istoken($key);
			$out = $key;
		}
		if(defined $val) {
			if(isnasty($val)) {
				$val =~ s/(["\\])/\\$1/g;
				$val = "\"$val\"";
			}
			
			$out .= '='
				if defined $key;

			$out .= $val;
		}
		push @out, $out;
	}

	return join('; ', @out);
}

use overload
	'""' => \&toString,
	fallback => 1;
